package ru.crystals.setapi.plugins;

/**
 * Перечисление возможных этапов/состояний техпроцесса добавления товарной позиции в чек продажи
 */
public enum SaleProcessState {

    /**
     * НАЧАЛО техпроцесса.
     * Состояние ввода номера телефона
     * Возможные переходы:
     * <og>
     * <li> {@link #PRINT_PRESALE_SLIP} - по событию успешного завершения ввода номера телефона;
     * <li> {@link #REJECT} - по нажатию клавиши "Отмена" на форме ввода номера телефона.
     * </og>
     */
    ENTER_PHONE_NUMBER,

    /**
     * Состояние печати предчека
     * Возможные переходы:
     * <og>
     * <li> {@link #PRESALE_DIALOG} - по событию успешного завершения печати предчека;
     * <li> {@link #PRINT_PRESALE_FAILED_DIALOG} - при возникновении ошибки при печати предчека.
     * </og>
     */
    PRINT_PRESALE_SLIP,

    /**
     * Состояние отображения диалога "Печать предчека не удалась. Повторить?"
     * Возможные переходы:
     * <og>
     * <li> {@link #PRINT_PRESALE_SLIP} - по выбору "Да, повторить" на диалоге;
     * <li> {@link #REJECT} - по выбору "Нет" или нажатии клавиши "Отмена" на диалоге.
     * </og>
     */
    PRINT_PRESALE_FAILED_DIALOG,

    /**
     * Состояние отображения диалога "корректна ли персональная информация на предчеке?"
     * Возможные переходы:
     * <og>
     * <li> {@link #ENTER_PHONE_NUMBER} - по выбору "нет" или нажатию клавиши "Отмена" на диалоге;
     * <li> {@link #ACTIVATE} - по выбору "да" на диалоге.
     * </og>
     */
    PRESALE_DIALOG,

    /**
     * Состояние активации товара в "процессинге" (с вытягиванием данных об этом товаре оттуда).
     * Возможные переходы (NOTE: негативный сценарий пока (2018-06-07) не реализован):
     * <og>
     * <li> {@link #SCAN_CARD_DIALOG} - безусловно по завершению процесса "активации".
     * </og>
     */
    ACTIVATE,

    /**
     * Состояние ожидания прокатывания карты через кард-ридер
     * Возможные переходы:
     * <og>
     * <li> {@link #ADD} - по успешному сканированию/вводу номера/прокатыванию карты;
     * <li> {@link #REJECT} - при отказе от сканирования карты
     * </og>
     * */
    SCAN_CARD_DIALOG,

    /**
     * Состояние непосредственного добавления уже сформированной позиции в чек.
     * Возможные переходы:
     * <og>
     * <li> {@link #FINISH_SUCCESS} - по успешному завершению добавления позиции в чек;
     * <li> {@link #ADDING_FAILED_MSG} - по возникновению ошибки при добавлении.
     * </og>
     */
    ADD,

    /**
     * Состояние отображения сообщения об ошибке, возникшей при добавлении позиции в чек
     * Возможные переходы:
     * <og>
     * <li> {@link #REJECT} - по нажатию любой клавиши на форме отображения сообщения об ошибке.
     * </og>
     */
    ADDING_FAILED_MSG,

    /**
     * Состояние отказа от добавления уже сформированной позиции в чек.
     * Возможные переходы:
     * <og>
     * <li> {@link #REJECT_FAILED_MSG} - по возникновению ошибки при отмене добавления позиции в чек;
     * <li> {@link #FINISH_FAILED} - если ошибки не возникло.
     * </og>
     */
    REJECT,

    /**
     * Состояние отображения сообщения об ошибке, возникшей при отказе от добавления позиции в чек
     * Возможные переходы:
     * <og>
     * <li> {@link #REJECT} - по нажатию любой клавиши на форме отображения сообщения об ошибке.
     * </og>
     */
    REJECT_FAILED_MSG,

    /**
     * ТЕРМИНАЛЬНОЕ состояние.
     * Завершение техпроцесса по позитивному сценарию (позиция в чек добавлена)
     */
    FINISH_SUCCESS,

    /**
     * ТЕРМИНАЛЬНОЕ состояние.
     * Завершение техпроцесса по по негативному сценарию (позиция в чек НЕ добавлена)
     */
    FINISH_FAILED;
}