package ru.crystals.pos.api.ext.loyal.dto;

import ru.crystals.pos.spi.receipt.LineItem;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

/**
 * Описывает результат обработки плагинами лояльности события окончательного расчета скидок.<br>
 * На этом этапе плагины лояльности могут подкорректировать у карт доступные потолки списания, а также добавить
 * какие-либо дополнительные атрибуты к чеку или позициям его.
 *
 * @since 0.0.39
 */
public class PostDiscountResult {

    /**
     * Хранит измененные потолки списания по картам.
     * Ключом служит номер карты, значением - потолок списания. Допустимо хранить null в качестве значения.
     */
    private Map<String, BigDecimal> bonusWriteOffLimits;

    /**
     * Мапа расширенных атрибутов чека.
     *
     * @since 0.0.55
     */
    private Map<String, String> receiptAttributeMap = new HashMap<>();

    /**
     * Мапа расширенных атрибутов позиции. Ключом в мапе служит порядковый номер позиции, который получается как
     * {@link LineItem#getNumber()}, значением в мапе - мапа идентификаторов атрибутов и значений их.
     *
     * @since 0.0.55
     */
    private Map<Integer, Map<String, String>> positionAttributeMap = new HashMap<>();

    /**
     * Возвращает мапу исправленных плагином потолков списания бонусов карт.<br>
     * Ключом в мапе служит номер карты, значением - максимально допустимая величина списания бонусов по данной карте.
     *
     * @return мапа максимальных величин списания бонусов по картам или пустая мапа, если таких нет.
     */
    public Map<String, BigDecimal> getBonusWriteOffLimits() {
        if (bonusWriteOffLimits == null) {
            bonusWriteOffLimits = new HashMap<>();
        }
        return bonusWriteOffLimits;
    }

    /**
     * Возвращает мапу расширенных атрибутов чека. Ключом в мапе служит произвольный строковый идентификатор атрибута,
     * значением - значение атрибута.
     *
     * @return мапа расширенных атрибутов чека.
     * @since 0.0.55
     */
    public Map<String, String> getReceiptAttributeMap() {
        return receiptAttributeMap;
    }

    /**
     * Возвращает мапу расширенных атрибутов позиций. Ключом в мапе служит номер позиции, получаемый как {@link LineItem#getNumber()},
     * значением - мапа атрибутов, которые присвоены данной позиции.
     *
     * @return мапа расширенных атрибутов позиций.
     * @see #addPositionAttribute(int, String, String)
     * @since 0.0.55
     */
    public Map<Integer, Map<String, String>> getPositionAttributeMap() {
        return positionAttributeMap;
    }

    /**
     * Добавляет к позиции с указанным номером заданный атрибут и его значение.
     *
     * @param number номер позиции значение нужно добавить к которой.
     * @param key    идентификатор атрибута.
     * @param value  значение атрибута.
     * @since 0.0.55
     */
    public void addPositionAttribute(int number, String key, String value) {
        positionAttributeMap.computeIfAbsent(number, k -> new HashMap<>());
        positionAttributeMap.get(number).put(key, value);
    }

    @Override
    public String toString() {
        return "PostDiscountResult{" +
                "bonusWriteOffLimits=" + bonusWriteOffLimits +
                ", receiptAttributeMap=" + receiptAttributeMap +
                ", positionAttributeMap=" + positionAttributeMap +
                '}';
    }
}
