package ru.crystals.pos.spi;

import java.math.BigDecimal;
import java.util.Map;

/**
 * Интерфейс для чтения настроек {@code<ключ, значение>}.
 * <p>
 * Поскольку параметры могут быть изменены централизованно в любой момент
 * времени, не рекомендуется хранить копии их значений в классе плагина.<br>
 * 
 * <h3>Рекомендуется:</h3>
 * 
 * <pre>
 * &#064;POSPlugin(id = "foo.plugin")
 * public class FooPlugin implements PaymentPlugin {
 * 
 *     &#064;Inject
 *     private IntegrationProperties properties;
 * 
 *     &#64;Override
 *     public void doPayment(PaymentRequest request) {
 *         String ip = properties.getServiceProperties().get("processing.ip");
 *         // do something
 *     }
 * 
 * }
 * </pre>
 * 
 * <h3>Не рекомендуется:</h3>
 * 
 * <pre>
 * &#064;POSPlugin(id = "foo.plugin")
 * public class FooPlugin implements PaymentPlugin {
 * 
 *     &#064;Inject
 *     private IntegrationProperties properties;
 * 
 *     private String ip = null;
 * 
 *     &#64;Override
 *     public void doPayment(PaymentRequest request) {
 *         if (ip == null) {
 *             String ip = properties.getServiceProperties().get("processing.ip");
 *         }
 *         // do something
 *     }
 * 
 * }
 * </pre>
 * 
 */
public interface PropertiesReader {

    /**
     * Получить все настройки.
     * @return 
     *      Коллекция всех имеющихся настроек.
     */
    Map<String, String> getAllProperties();

    /**
     * Чтение символьного параметра.
     * 
     * @param key
     *            ключ параметра
     * @return Значение параметра, null если параметр отсутствует
     */
    String get(String key);

    /**
     * Чтение символьного параметра.
     * 
     * @param key
     *            ключ параметра
     * @param defaultValue
     *            значение по умолчанию
     * @return Значение параметра, если он указан, иначе <b>defaultValue</b>
     */
    String get(String key, String defaultValue);

    /**
     * Чтение целочисленного параметра.
     * 
     * @param key
     *            ключ параметра
     * @return Значение параметра, null если параметр отсутствует
     */
    Integer getInt(String key);

    /**
     * Чтение целочисленного параметра.
     * 
     * @param key
     *            ключ параметра
     * @param defaultValue
     *            значение по умолчанию
     * @return Значение параметра, если он указан, иначе <b>defaultValue</b>
     */
    int getInt(String key, int defaultValue);

    /**
     * Чтение параметра, как массива байтов.
     * 
     * @param key
     *            ключ параметра
     * @return массив байтов, null если параметр отсутствует
     */
    byte[] getBytes(String key);

    /**
     * Чтение логического параметра.
     * 
     * @param key
     *            ключ параметра
     * @return Значение параметра, если он указан, иначе <b>false</b>
     */
    boolean getBoolean(String key);

    /**
     * Чтение логического параметра.
     * 
     * @param key
     *            ключ параметра
     * @param defaultValue
     *            значение по умолчанию
     * @return Значение параметра, если он указан, иначе <b>false</b>
     */
    boolean getBoolean(String key, boolean defaultValue);

    /**
     * Возвращает из настроек значение как {@link BigDecimal} параметра с указанным идентификатором.
     * @param key идентификатор параметра, значение которого требуется прочитать как {@link BigDecimal}.
     * @return значение параметра в {@link BigDecimal} или null, если такого нет или значение его не может быть приведено к данному типу.
     * @since 0.0.35
     */
    BigDecimal getDecimal(String key);

    /**
     * Возвращает из настроек значение как {@link BigDecimal} параметра с указанным идентификатором.
     * @param key идентификатор параметра, значение которого требуется прочитать как {@link BigDecimal}.
     * @param defaultValue значение параметра по умолчанию на случай, если его значение не задано или не может быть приведено к {@link BigDecimal}
     * @return значение параметра в {@link BigDecimal} или значение аргумента defaultValue, если значение параметра не задано или не может быть приведено к данному типу.
     * @since 0.0.35
     */
    BigDecimal getDecimal(String key, BigDecimal defaultValue);

    /**
     * Сохранить настройки.
     * @param properties коллекция настроек для сохранения.
     */
    void setAllProperties(Map<String, String> properties);

    /**
     * Запись символьного параметра.
     * @param key ключ параметра
     * @param value значение параметра
     */
    void set(String key, String value);

    /**
     * Запись целочисленного параметра.
     * @param key ключ параметра
     * @param value значение параметра
     */
    void setInt(String key, Integer value);

    /**
     * Запись параметра как массива байтов.
     * @param key ключ параметра
     * @param value значение параметра
     */
    void setBytes(String key, byte[] value);

    /**
     * Запись логического параметра.
     * @param key ключ параметра
     * @param value значение параметра
     */
    void setBoolean(String key, Boolean value);

    /**
     * Сохраняет значение параметра как {@link BigDecimal}.
     * @param key идентификатор параметра.
     * @param value значение параметра.
     */
    void setDecimal(String key, BigDecimal value);
}
